import { useEffect, useState } from 'react';
import { Pet } from '../../types/Pet';
import { Skill, PetSkill, SkillType } from '../../types/Skills';
import { fetchPets, getAllSkills, getPetSkills, postAllocatePetSkill } from '../../services/api/api';
import { Loader2 } from 'lucide-react';
import styles from './SkillTreeModal.module.css';

interface SkillTreeModalProps {
  onClose: () => void;
  petId: string;
  onPetUpdate: (updatedPet: Pet) => void;
}

export default function SkillTreeModal({ onClose, petId, onPetUpdate }: SkillTreeModalProps) {
  const [skills, setSkills] = useState<Skill[]>([]);
  const [petSkills, setPetSkills] = useState<PetSkill[]>([]);
  const [loading, setLoading] = useState(true);
  const [allocating, setAllocating] = useState(false);

  useEffect(() => {
    const fetchData = async () => {
      try {
        const [allSkills, ownedSkills] = await Promise.all([
          getAllSkills(),
          getPetSkills(petId)
        ]);
        setSkills(allSkills);
        setPetSkills(ownedSkills);
      } catch (error) {
        console.error('Error fetching skills:', error);
      } finally {
        setLoading(false);
      }
    };
    fetchData();
  }, [petId]);

  const handleAllocateSkill = async (skillId: number) => {
    try {
      setAllocating(true);
      const updatedPetSkill = await postAllocatePetSkill(petId, skillId);
      setPetSkills([...petSkills, updatedPetSkill]);
      // Refresh pet data to update skill points
      const updatedPet = await fetchPets();
      onPetUpdate(updatedPet[0]);
    } catch (error) {
      console.error('Error allocating skill:', error);
    } finally {
      setAllocating(false);
    }
  };

  const getSkillTier = (skillId: number) => {
    return petSkills.find(ps => ps.skillId === skillId)?.currentTier;
  };

  const canAllocateSkill = (skill: Skill) => {
    if (!skill.skillsIdRequired) return true;
    return petSkills.some(ps => ps.skillId === skill.skillsIdRequired![0]);
  };

  const getRequiredSkillName = (skillId: number[] | null) => {
    if (!skillId) return null;
    return skills.find(s => s.id === skillId[0])?.name;
  };

  const renderSkillNode = (skill: Skill) => {
    console.log('Rendering skill:', skill);

    const owned = petSkills.some(ps => ps.skillId === skill.id);
    const tier = getSkillTier(skill.id);
    const canAllocate = canAllocateSkill(skill);
    const requiredSkillName = getRequiredSkillName(skill.skillsIdRequired);

    return (
      <div
        key={skill.id}
        className={`
          relative p-4 rounded-lg border-2
          ${owned ? 'border-green-500 bg-gray-700' : 'border-gray-600 bg-gray-800'}
          ${!owned && !canAllocate ? 'opacity-50' : ''}
          hover:border-blue-500 transition-colors
        `}
      >
        <div className="flex items-center gap-2 mb-2">
          {/* <img src={skill.icon} alt={skill.name} className="w-6 h-6" /> */}
          <span>{skill.icon}</span>
          <h3 className="font-bold">{skill.name}</h3>
          {tier && (
            <span className="px-2 py-1 bg-blue-600 rounded text-xs">
              Tier {tier}
            </span>
          )}
        </div>
        <p className="text-sm text-gray-300 mb-2">{skill.description}</p>
        {!canAllocate && requiredSkillName && (
          <p className="text-red-500 text-sm mb-2">Required skill: {requiredSkillName}</p>
        )}
        <div className="text-xs text-gray-400">
          {skill.effects.map((effect, idx) => (
            <div key={idx}>
              Tier {effect.tier}: {effect.effect} ({effect.value})
            </div>
          ))}
        </div>
        {!owned && (
          <button
            onClick={() => handleAllocateSkill(skill.id)}
            disabled={!canAllocate || allocating}
            className={`
              mt-2 px-3 py-1 rounded text-sm
              ${canAllocate
                ? 'bg-blue-600 hover:bg-blue-700'
                : 'bg-gray-600 cursor-not-allowed'}
            `}
          >
            {allocating ? (
              <Loader2 className="w-4 h-4 animate-spin" />
            ) : (
              `Learn (${skill.pointsCost} SP)`
            )}
          </button>
        )}
        {owned && (
          <button
            onClick={() => handleAllocateSkill(skill.id)}
            disabled={!canAllocate || allocating}
            className={`
              mt-2 px-3 py-1 rounded text-sm
              ${canAllocate
                ? 'bg-green-600 hover:bg-green-700'
                : 'bg-gray-600 cursor-not-allowed'}
            `}
          >
            {allocating ? (
              <Loader2 className="w-4 h-4 animate-spin" />
            ) : (
              `Upgrade (${skill.pointsCost} SP)`
            )}
          </button>
        )}
      </div>
    );
  };

  const renderSkillTree = () => {
    const groundSkills = skills.filter(s => s.type === 'GROUND');
    const grandSkills = skills.filter(s => s.type === 'GRAND');

    return (
      <div className="space-y-6">
        <div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 gap-4">
          {groundSkills.map(renderSkillNode)}
        </div>
        <div className="border-t border-gray-600 pt-6">
          <h2 className="text-xl font-bold mb-4">Advanced Skills</h2>
          <div className="grid grid-cols-1 md:grid-cols-2 gap-4">
            {grandSkills.map(renderSkillNode)}
          </div>
        </div>
      </div>
    );
  };

  return (
    <div className="fixed inset-0 bg-black/50 backdrop-blur-sm flex items-center justify-center p-4 z-50">
      <div className={`bg-gray-800 p-6 rounded-xl max-w-4xl w-full max-h-[90vh] overflow-y-auto ${styles.customScroll}`}>
        <h2 className="text-2xl font-bold mb-4">Skill Tree</h2>

        {loading ? (
          <div className="flex justify-center items-center h-40">
            <Loader2 className="w-8 h-8 animate-spin" />
          </div>
        ) : (
          renderSkillTree()
        )}

        <div className="mt-6 flex justify-end">
          <button
            onClick={onClose}
            className="bg-red-600 hover:bg-red-700 text-white px-4 py-2 rounded"
          >
            Close
          </button>
        </div>
      </div>
    </div>
  );
}
